# Copyright (c) 2010 Aldo Cortesi
# Copyright (c) 2010, 2014 dequis
# Copyright (c) 2012 Randall Ma
# Copyright (c) 2012-2014 Tycho Andersen
# Copyright (c) 2012 Craig Barnes
# Copyright (c) 2013 horsik
# Copyright (c) 2013 Tao Sauvage
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
import libqtile
import os
from libqtile.backend.wayland import InputConfig
from libqtile import bar, layout, qtile, widget
from libqtile.config import Click, Drag, Group, Key, Match, Screen
from libqtile.widget import backlight
from libqtile.lazy import lazy
from libqtile.log_utils import logger
from layouts.tabbed import Tabbed
from qtile_bonsai import Bonsai
from qtile_extras.layout.decorations import RoundedCorners
#from libqtile.utils import guess_terminal
from libqtile import hook


mod = "mod4"
terminal = "kitty"
browser = "xdg-open http://"
wallpaper_path = os.path.expanduser("~/.wallpaper")
wallpaper_mode = "fill"

bind_windows = {
    "1": [
        "kitty"
    ],
    "2": [
        "zen",
        "firefox"
    ],
    "3": [
        "dev.zed.Zed",
        "Code"
    ],
    "4": [],
    "5": [
        "Element"
    ]
}

# Switches between tabbed and plasma layout
@lazy.function
def switch_tabbed(qtile):
    tabbed_layout = plasma_layout = None
    for index, obj in enumerate(qtile.current_group.layouts):
        if obj.name == "tabbed":
            tabbed_layout = index
        elif obj.name == "plasma":
            plasma_layout = index
    if qtile.current_group.layout.name == "tabbed":
        qtile.current_group.use_layout(plasma_layout)
    else:
        qtile.current_group.use_layout(tabbed_layout)

keys = [
    # A list of available commands that can be bound to keys can be found
    # at https://docs.qtile.org/en/latest/manual/config/lazy.html
    # Switch between windows
    Key([mod], "Left", lazy.layout.left(), desc="Move focus to left"),
    Key([mod], "Right", lazy.layout.right(), desc="Move focus to right"),
    Key([mod], "Down", lazy.layout.down(), desc="Move focus down"),
    Key([mod], "Up", lazy.layout.up(), desc="Move focus up"),
    Key([mod], "space", lazy.layout.next(), desc="Move window focus to other window"),
    # Move windows between left/right columns or move up/down in current stack.
    # Moving out of range in Columns layout will create new column.
    Key([mod, "shift"], "Left", lazy.layout.move_left(), desc="Move window to the left"),
    Key([mod, "shift"], "Right", lazy.layout.move_right(), desc="Move window to the right"),
    Key([mod, "shift"], "Down", lazy.layout.move_down(), desc="Move window down"),
    Key([mod, "shift"], "Up", lazy.layout.move_up(), desc="Move window up"),
    # Grow windows. If current window is on the edge of screen and direction
    # will be to screen edge - window would shrink.
    Key([mod, "control"], "Left", lazy.layout.grow_width(-30), desc="Grow window to the left"),
    Key([mod, "control"], "Right", lazy.layout.grow_width(30), desc="Grow window to the right"),
    Key([mod, "control"], "Down", lazy.layout.grow_height(-30), desc="Grow window down"),
    Key([mod, "control"], "Up", lazy.layout.grow_height(30), desc="Grow window up"),
    Key([mod], "n", lazy.layout.reset_size(), desc="Reset all window sizes"),
    # Toggle between split and unsplit sides of stack.
    # Split = all windows displayed
    # Unsplit = 1 window displayed, like Max layout, but still with
    # multiple stack panes
    Key(
        [mod, "shift"],
        "Return",
        lazy.layout.toggle_split(),
        desc="Toggle between split and unsplit sides of stack",
    ),
    Key([mod], "Return", lazy.spawn(terminal), desc="Launch terminal"),
    Key([mod], "w", lazy.spawn(browser), desc="Launch Browser"),
    # Toggle between different layouts as defined below
    Key([mod], "Tab", lazy.next_layout(), desc="Toggle between layouts"),
    Key([mod], "t", switch_tabbed, desc="Switch to tabbed layout"),
    Key([mod], "q", lazy.window.kill(), desc="Kill focused window"),
    Key(
        [mod],
        "f",
        lazy.window.toggle_fullscreen(),
        desc="Toggle fullscreen on the focused window",
    ),
    Key([mod, "shift"], "space", lazy.window.toggle_floating(), desc="Toggle floating on the focused window"),
    Key([mod, "shift"], "r", lazy.reload_config(), desc="Reload the config"),
    Key([mod, "shift"], "q", lazy.shutdown(), desc="Shutdown Qtile"),
    Key([mod], "r", lazy.spawncmd(), desc="Spawn a command using a prompt widget"),
    Key(
        [],
        "XF86MonBrightnessUp",
        lazy.widget["backlight"].change_backlight(backlight.ChangeDirection.UP)
    ),
    Key(
        [],
        "XF86MonBrightnessDown",
        lazy.widget["backlight"].change_backlight(backlight.ChangeDirection.DOWN)
    ),
    Key(
        [],
        "XF86AudioRaiseVolume",
        lazy.spawn("pamixer -i 5")
    ),
    Key(
        [],
        "XF86AudioLowerVolume",
        lazy.spawn("pamixer -d 5")
    ),
    Key(
        [],
        "XF86AudioMute",
        lazy.spawn("pamixer -t")
    ),
    Key(
        [],
        "XF86AudioPrev",
        lazy.spawn("playerctl previous")
    ),
    Key(
        [],
        "XF86AudioPlay",
        lazy.spawn("playerctl play-pause")
    ),
    Key(
        [],
        "XF86AudioNext",
        lazy.spawn("playerctl next")
    )
]

# Add key bindings to switch VTs in Wayland.
# We can't check qtile.core.name in default config as it is loaded before qtile is started
# We therefore defer the check until the key binding is run by using .when(func=...)
for vt in range(1, 8):
    keys.append(
        Key(
            ["control", "mod1"],
            f"f{vt}",
            lazy.core.change_vt(vt).when(func=lambda: qtile.core.name == "wayland"),
            desc=f"Switch to VT{vt}",
        )
    )


groups = [Group(i) for i in "123456789"]

for i in groups:
    keys.extend(
        [
            # mod + group number = switch to group
            Key(
                [mod],
                i.name,
                lazy.group[i.name].toscreen(),
                desc=f"Switch to group {i.name}",
            ),
            # mod + shift + group number = switch to & move focused window to group
            Key(
                [mod, "shift"],
                i.name,
                lazy.window.togroup(i.name, switch_group=True),
                desc=f"Switch to & move focused window to group {i.name}",
            ),
            # Or, use below if you prefer not to switch to that group.
            # # mod + shift + group number = move focused window to group
            # Key([mod, "shift"], i.name, lazy.window.togroup(i.name),
            #     desc="move focused window to group {}".format(i.name)),
        ]
    )

layouts = [
    layout.Plasma(
        border_focus=RoundedCorners(colour="4C7899"),
        border_normal=RoundedCorners(colour="333333"),
        border_width=3,
        border_width_single=3,
        margin=2
    ),
    Tabbed(),
    layout.RatioTile(),
    Bonsai(),
    # layout.Columns(border_focus_stack=["#d75f5f", "#8f3d3d"], border_width=4),
    layout.Max(),
    # Try more layouts by unleashing below layouts.
    # layout.Stack(num_stacks=2),
    # layout.Bsp(),
    # layout.Matrix(),
    # layout.MonadTall(),
    # layout.MonadWide(),
    # layout.Tile(),
    layout.TreeTab(),
    # layout.VerticalTile(),
    layout.Zoomy(),
]

widget_defaults = dict(
    font="JetBrainsMonoNerdFont",
    fontsize=12,
    padding=3,
)
extension_defaults = widget_defaults.copy()

screens = [
    Screen(
        wallpaper=wallpaper_path,
        wallpaper_mode=wallpaper_mode,
        top=bar.Bar(
            [
                widget.CurrentLayout(),
                widget.GroupBox(),
                widget.Prompt(),
                widget.WindowName(
                    max_chars=25
                ),
                widget.Chord(
                    chords_colors={
                        "launch": ("#ff0000", "#ffffff"),
                    },
                    name_transform=lambda name: name.upper(),
                ),
                widget.Mpris2(
                    format="{xesam:title} - {xesam:artist}",
                    max_chars=30
                ),
                widget.Battery(
                    format="Battery: {percent:2.0%} ({char})",
                    empty_char="Empty",
                    full_char="Full",
                    charge_char="Charging",
                    discharge_char="Discharging",
                    unknown_char="Unknown",
                    not_charging_char="Not Charging",
                    notify_below=15,
                    update_interval=1,
                    show_short_text=False,
                    hide_crash=True
                ),
                widget.Volume(
                    get_volume_command="pamixer --get-volume-human",
                    volume_app="pavucontrol",
                    volume_up_command="pamixer -i 1",
                    volume_down_command="pamixer -d 1"

                ),
                widget.Backlight(change_command="brightnessctl set {0}%", step=5, format="", hide_crash=True),
                widget.TextBox("default config", name="default"),
                widget.TextBox("Press &lt;M-r&gt; to spawn", foreground="#d75f5f"),
                # NB Systray is incompatible with Wayland, consider using StatusNotifier instead
                # widget.StatusNotifier(),
                widget.Systray(),
                widget.Clock(format="%d.%m.%Y %X"),
                widget.QuickExit(),
            ],
            24,
            background="#32323232"
            # border_width=[2, 0, 2, 0],  # Draw top and bottom borders
            # border_color=["ff00ff", "000000", "ff00ff", "000000"]  # Borders are magenta
        ),
        # You can uncomment this variable if you see that on X11 floating resize/moving is laggy
        # By default we handle these events delayed to already improve performance, however your system might still be struggling
        # This variable is set to None (no cap) by default, but you can set it to 60 to indicate that you limit it to 60 events per second
        # x11_drag_polling_rate = 60,
    ),
]

# Drag floating layouts.
mouse = [
    Drag([mod], "Button1", lazy.window.set_position(), start=lazy.window.get_position()),
    Drag([mod], "Button3", lazy.window.set_size(), start=lazy.window.get_size()),
    Click([mod], "Button2", lazy.window.bring_to_front()),
]

dgroups_key_binder = None
dgroups_app_rules = []  # type: list
follow_mouse_focus = True
bring_front_click = False
floats_kept_above = True
cursor_warp = False
floating_layout = layout.Floating(
    float_rules=[
        # Run the utility of `xprop` to see the wm class and name of an X client.
        *layout.Floating.default_float_rules,
        Match(wm_class="confirmreset"),  # gitk
        Match(wm_class="makebranch"),  # gitk
        Match(wm_class="maketag"),  # gitk
        Match(wm_class="ssh-askpass"),  # ssh-askpass
        Match(title="branchdialog"),  # gitk
        Match(title="pinentry"),  # GPG key password entry
    ]
)


@hook.subscribe.client_new
def bind_group(spawned_window):
    moved = False
    for group, windows in bind_windows.items():
        for window in windows:
            if not moved and spawned_window.get_wm_class() is not None and window in spawned_window.get_wm_class():
                spawned_window.togroup(group, switch_group=True)
                moved = True
    if not moved:
        logger.warning(f"Window with Classname(s) {spawned_window.get_wm_class()} is not assigned to a group")

auto_fullscreen = True
focus_on_window_activation = "smart"
reconfigure_screens = True

# If things like steam games want to auto-minimize themselves when losing
# focus, should we respect this or not?
auto_minimize = True

# When using the Wayland backend, this can be used to configure input devices.
wl_input_rules = {
    "type:keyboard": InputConfig(kb_layout="de", kb_variant="mac"),
    "type:touchpad": InputConfig(natural_scroll=True, tap=True),
    "type:pointer": InputConfig(natural_scroll=True)
}

# xcursor theme (string or None) and size (integer) for Wayland backend
wl_xcursor_theme = None
wl_xcursor_size = 24

# XXX: Gasp! We're lying here. In fact, nobody really uses or cares about this
# string besides java UI toolkits; you can see several discussions on the
# mailing lists, GitHub issues, and other WM documentation that suggest setting
# this string if your java app doesn't work correctly. We may as well just lie
# and say that we're a working one by default.
#
# We choose LG3D to maximize irony: it is a 3D non-reparenting WM written in
# java that happens to be on java's whitelist.
wmname = "LG3D"
